<?php

/*
 *  Useful functions
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 */

/*
 *  Bidirectional Bubblesort ("Shakersort") which can sort an array
 *  by a sub key. This is useful for result arrays returned by the
 *  vmailmgr interface. The array structure looks like this:
 *
 *    $a = array(
 *      array(
 *        "name" => "Foo",
 *        "mail" => "foo@moo.net"
 *      ),
 *      array(
 *        "name" => "Bar",
 *        "mail" => "bar@bie.com"
 *      )
 *    );
 *
 *    array_subkey_sort($a, "name");
 *
 *  Same thing in reverse order:
 *
 *    array_subkey_rsort($a, "name");
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 *
 */

/* patched string comparison: usually an empty string is greater
   than any other string it is compared to, but for vmmi it is 
   preferrable to have all empty strings at the end. These "patched"
   string comparisons will provide this behaviour */

function is_lexically_greater($a, $b, $patch) {
  if($patch) {
    if($a == $b)
      return false;
    else if($a=='')
      return true;
    else if($b=='')
      return false;
  }
  
  return $a > $b;
}

function is_lexically_less($a, $b, $patch) {
  if($patch) {
    if($a == $b)
      return false;
    else if($a=='')
      return false;
    else if($b=='')
      return true;
  }
  
  return $a < $b;
}

function array_swap(&$array, $element1, $element2) {
  $tmp = $array[$element1];
  $array[$element1] = $array[$element2];
  $array[$element2] = $tmp;
}

function array_subkey_sort(&$array, $subkey, $patch=true) {
  $lower = 0;
  $upper = count($array) - 1;

  while($lower != $upper) {
    $last = $lower;
    for($pos = $lower; $pos < $upper; ++$pos) {
      if(is_lexically_greater($array[$pos][$subkey], $array[$pos + 1][$subkey], $patch)) {
        array_swap($array, $pos, $pos + 1);
	$max = $pos;
      }
    }
    $upper = $max;

    $last = $upper;
    for($pos = $upper; $pos > $lower; --$pos) {
      if(is_lexically_less($array[$pos][$subkey], $array[$pos - 1][$subkey], $patch)) {
        array_swap($array, $pos, $pos - 1);
	$max = $pos;
      }
    }
    $lower = $max;
  }
}

function array_subkey_rsort(&$array, $subkey) {
  array_subkey_sort($array, $subkey, false);
  $array = array_reverse($array);
}

/*
 *  Compare every single word inside two strings using soundex()
 *  also considering german umlauts. This is meant to imitate the
 *  behaviour of popular search engines: intuitive and accurate.
 *  Boolean expressions (AND, OR, ..) are outdated.
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 */

function is_similar($a, $b) {
  if($a == $b)
    return true;

  if(trim($a) == '' || trim($b) == '')
    return false;

  $umlauts = array(
    '' => 'ae',
    '' => 'oe',
    '' => 'ue',
    '' => 'Ae',
    '' => 'Oe',
    '' => 'Ue'
  );

  foreach($umlauts as $from => $to) {
    $a = str_replace($from, $to, $a);
    $b = str_replace($from, $to, $b);
  }

  $list_a = preg_split("/ +|-/", $a);
  $list_b = preg_split("/ +|-/", $b);
  
  for($x=0; $x<count($list_a); ++$x) {
    for($y=0; $y<count($list_b); ++$y) {
      if(soundex($list_a[$x]) == soundex($list_b[$y]))
        return true;
    }
  }  
  
  return false;
}

/*
 *  Create a short session-key
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 */
 
function make_session_key($length=10) {
  return substr(md5(microtime()), 0, $length);
}

/*
 *  Build GET-parameterlist   
 *
 *    build_query(array(
 *      'foo' => 'bar',
 *      'moo' => 'cow'
 *    ), array(
 *      'sort',
 *      'align'
 *    ));
 *
 *  returns
 *
 *    foo=bar&moo=cow&sort=asc&align=left
 *
 *  while 'sort' and 'align' are $_GET params which already existed.
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 */
 
function build_query($new, $preserve=array()) {
  foreach($new as $key => $val) {
    $query[] = $key . "=" . $val;
  }
  
  for($i=0; $i<count($preserve); ++$i) {
    if(isset($_GET[$preserve[$i]]) && $_GET[$preserve[$i]] != '') {
      $query[] = $preserve[$i] . "=" . rawurlencode($_GET[$preserve[$i]]);
    }
  }
    
  return @join("&", $query);
}

/*
 *  format_filesize choses the best human readable unit for $bytes returns a formatted string
 *  Updates, documentation and examples: http://daniel.lorch.cc/projects/format_filesize/
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 */

function format_filesize($bytes, $decimals = 1)
{
    $units = array(
      '1152921504606846976' => 'EB', /* Exa Byte  10^18 */
      '1125899906842624'    => 'PB', /* Peta Byte 10^15 */
      '1099511627776'       => 'TB',
      '1073741824'          => 'GB',
      '1048576'             => 'MB',
      '1024'                => 'KB'
    );

    if($bytes <= 1024)
      return $bytes . " Bytes";
    
    foreach($units as $base => $title)
      if(floor($bytes / $base) != 0)
        return number_format($bytes / $base, $decimals, ".", "'") . ' ' . $title;
}

/*
 *  Symmetric encryption using RC4 algorithm.
 *
 *  Borrowed from PEAR
 *
 */
 
include dirname(__FILE__) . '/Crypt_Rc4.php';

/* write wrappers around it */

function rc4_encrypt($key, $message) {
  $rc4 = new Crypt_RC4();
  $rc4->key($key);
  $rc4->crypt($message);
  
  return $message;
}

function rc4_decrypt($key, $message) {
  $rc4 = new Crypt_RC4();
  $rc4->key($key);
  $rc4->decrypt($message);
  
  return $message;
}

?>
